@@ -10,6 +10,7 @@ from agno.db.base import SessionType
 from agno.models.message import Message
 from agno.os.config import ChatConfig, EvalsConfig, KnowledgeConfig, MemoryConfig, MetricsConfig, SessionConfig
 from agno.os.utils import (
+    extract_input_media,
     format_team_tools,
     format_tools,
     get_run_input,
@@ -461,11 +462,8 @@ class TeamResponse(BaseModel):
             "stream_member_events": False,
         }
 
-        if team.model is None:
-            raise ValueError("Team model is required")
-
         team.determine_tools_for_model(
-            model=team.model,
+            model=team.model,  # type: ignore
             session=TeamSession(session_id=str(uuid4()), session_data={}),
             run_response=TeamRunOutput(run_id=str(uuid4())),
             async_mode=True,
@@ -763,8 +761,10 @@ class TeamSessionDetailSchema(BaseModel):
     session_state: Optional[dict]
     metrics: Optional[dict]
     team_data: Optional[dict]
+    chat_history: Optional[List[dict]]
     created_at: Optional[datetime]
     updated_at: Optional[datetime]
+    total_tokens: Optional[int]
 
     @classmethod
     def from_session(cls, session: TeamSession) -> "TeamSessionDetailSchema":
@@ -783,6 +783,7 @@ class TeamSessionDetailSchema(BaseModel):
             if session.session_data
             else None,
             metrics=session.session_data.get("session_metrics", {}) if session.session_data else None,
+            chat_history=[message.to_dict() for message in session.get_chat_history()],
             created_at=datetime.fromtimestamp(session.created_at, tz=timezone.utc) if session.created_at else None,
             updated_at=datetime.fromtimestamp(session.updated_at, tz=timezone.utc) if session.updated_at else None,
         )
@@ -826,17 +827,27 @@ class WorkflowSessionDetailSchema(BaseModel):
 
 class RunSchema(BaseModel):
     run_id: str
-    agent_session_id: Optional[str]
+    parent_run_id: Optional[str]
+    agent_id: Optional[str]
     user_id: Optional[str]
     run_input: Optional[str]
     content: Optional[Union[str, dict]]
     run_response_format: Optional[str]
     reasoning_content: Optional[str]
+    reasoning_steps: Optional[List[dict]]
     metrics: Optional[dict]
     messages: Optional[List[dict]]
     tools: Optional[List[dict]]
     events: Optional[List[dict]]
     created_at: Optional[datetime]
+    references: Optional[List[dict]]
+    reasoning_messages: Optional[List[dict]]
+    images: Optional[List[dict]]
+    videos: Optional[List[dict]]
+    audio: Optional[List[dict]]
+    files: Optional[List[dict]]
+    response_audio: Optional[dict]
+    input_media: Optional[Dict[str, Any]]
 
     @classmethod
     def from_dict(cls, run_dict: Dict[str, Any]) -> "RunSchema":
@@ -844,16 +855,26 @@ class RunSchema(BaseModel):
         run_response_format = "text" if run_dict.get("content_type", "str") == "str" else "json"
         return cls(
             run_id=run_dict.get("run_id", ""),
-            agent_session_id=run_dict.get("session_id", ""),
+            parent_run_id=run_dict.get("parent_run_id", ""),
+            agent_id=run_dict.get("agent_id", ""),
             user_id=run_dict.get("user_id", ""),
             run_input=run_input,
             content=run_dict.get("content", ""),
             run_response_format=run_response_format,
             reasoning_content=run_dict.get("reasoning_content", ""),
+            reasoning_steps=run_dict.get("reasoning_steps", []),
             metrics=run_dict.get("metrics", {}),
             messages=[message for message in run_dict.get("messages", [])] if run_dict.get("messages") else None,
             tools=[tool for tool in run_dict.get("tools", [])] if run_dict.get("tools") else None,
             events=[event for event in run_dict["events"]] if run_dict.get("events") else None,
+            references=run_dict.get("references", []),
+            reasoning_messages=run_dict.get("reasoning_messages", []),
+            images=run_dict.get("images", []),
+            videos=run_dict.get("videos", []),
+            audio=run_dict.get("audio", []),
+            files=run_dict.get("files", []),
+            response_audio=run_dict.get("response_audio", None),
+            input_media=extract_input_media(run_dict),
             created_at=datetime.fromtimestamp(run_dict.get("created_at", 0), tz=timezone.utc)
             if run_dict.get("created_at") is not None
             else None,
@@ -863,8 +884,10 @@ class RunSchema(BaseModel):
 class TeamRunSchema(BaseModel):
     run_id: str
     parent_run_id: Optional[str]
+    team_id: Optional[str]
     content: Optional[Union[str, dict]]
     reasoning_content: Optional[str]
+    reasoning_steps: Optional[List[dict]]
     run_input: Optional[str]
     run_response_format: Optional[str]
     metrics: Optional[dict]
@@ -872,6 +895,14 @@ class TeamRunSchema(BaseModel):
     messages: Optional[List[dict]]
     events: Optional[List[dict]]
     created_at: Optional[datetime]
+    references: Optional[List[dict]]
+    reasoning_messages: Optional[List[dict]]
+    input_media: Optional[Dict[str, Any]]
+    images: Optional[List[dict]]
+    videos: Optional[List[dict]]
+    audio: Optional[List[dict]]
+    files: Optional[List[dict]]
+    response_audio: Optional[dict]
 
     @classmethod
     def from_dict(cls, run_dict: Dict[str, Any]) -> "TeamRunSchema":
@@ -880,10 +911,12 @@ class TeamRunSchema(BaseModel):
         return cls(
             run_id=run_dict.get("run_id", ""),
             parent_run_id=run_dict.get("parent_run_id", ""),
+            team_id=run_dict.get("team_id", ""),
             run_input=run_input,
             content=run_dict.get("content", ""),
             run_response_format=run_response_format,
             reasoning_content=run_dict.get("reasoning_content", ""),
+            reasoning_steps=run_dict.get("reasoning_steps", []),
             metrics=run_dict.get("metrics", {}),
             messages=[message for message in run_dict.get("messages", [])] if run_dict.get("messages") else None,
             tools=[tool for tool in run_dict.get("tools", [])] if run_dict.get("tools") else None,
@@ -891,12 +924,22 @@ class TeamRunSchema(BaseModel):
             created_at=datetime.fromtimestamp(run_dict.get("created_at", 0), tz=timezone.utc)
             if run_dict.get("created_at") is not None
             else None,
+            references=run_dict.get("references", []),
+            reasoning_messages=run_dict.get("reasoning_messages", []),
+            images=run_dict.get("images", []),
+            videos=run_dict.get("videos", []),
+            audio=run_dict.get("audio", []),
+            files=run_dict.get("files", []),
+            response_audio=run_dict.get("response_audio", None),
+            input_media=extract_input_media(run_dict),
         )
 
 
 class WorkflowRunSchema(BaseModel):
     run_id: str
     run_input: Optional[str]
+    events: Optional[List[dict]]
+    workflow_id: Optional[str]
     user_id: Optional[str]
     content: Optional[Union[str, dict]]
     content_type: Optional[str]
@@ -905,6 +948,15 @@ class WorkflowRunSchema(BaseModel):
     step_executor_runs: Optional[list[dict]]
     metrics: Optional[dict]
     created_at: Optional[int]
+    reasoning_content: Optional[str]
+    reasoning_steps: Optional[List[dict]]
+    references: Optional[List[dict]]
+    reasoning_messages: Optional[List[dict]]
+    images: Optional[List[dict]]
+    videos: Optional[List[dict]]
+    audio: Optional[List[dict]]
+    files: Optional[List[dict]]
+    response_audio: Optional[dict]
 
     @classmethod
     def from_dict(cls, run_response: Dict[str, Any]) -> "WorkflowRunSchema":
@@ -912,6 +964,8 @@ class WorkflowRunSchema(BaseModel):
         return cls(
             run_id=run_response.get("run_id", ""),
             run_input=run_input,
+            events=run_response.get("events", []),
+            workflow_id=run_response.get("workflow_id", ""),
             user_id=run_response.get("user_id", ""),
             content=run_response.get("content", ""),
             content_type=run_response.get("content_type", ""),
@@ -920,6 +974,15 @@ class WorkflowRunSchema(BaseModel):
             step_results=run_response.get("step_results", []),
             step_executor_runs=run_response.get("step_executor_runs", []),
             created_at=run_response["created_at"],
+            reasoning_content=run_response.get("reasoning_content", ""),
+            reasoning_steps=run_response.get("reasoning_steps", []),
+            references=run_response.get("references", []),
+            reasoning_messages=run_response.get("reasoning_messages", []),
+            images=run_response.get("images", []),
+            videos=run_response.get("videos", []),
+            audio=run_response.get("audio", []),
+            files=run_response.get("files", []),
+            response_audio=run_response.get("response_audio", None),
         )
 
 
@@ -936,6 +999,7 @@ class PaginationInfo(BaseModel):
     limit: Optional[int] = 20
     total_pages: Optional[int] = 0
     total_count: Optional[int] = 0
+    search_time_ms: Optional[float] = 0
 
 
 class PaginatedResponse(BaseModel, Generic[T]):
