@@ -12,12 +12,13 @@ class CsvTools(Toolkit):
         self,
         csvs: Optional[List[Union[str, Path]]] = None,
         row_limit: Optional[int] = None,
-        read_csvs: bool = True,
-        list_csvs: bool = True,
-        query_csvs: bool = True,
-        read_column_names: bool = True,
         duckdb_connection: Optional[Any] = None,
         duckdb_kwargs: Optional[Dict[str, Any]] = None,
+        enable_read_csv_file: bool = True,
+        enable_list_csv_files: bool = True,
+        enable_get_columns: bool = True,
+        enable_query_csv_file: bool = True,
+        all: bool = False,
         **kwargs,
     ):
         self.csvs: List[Path] = []
@@ -34,13 +35,13 @@ class CsvTools(Toolkit):
         self.duckdb_kwargs: Optional[Dict[str, Any]] = duckdb_kwargs
 
         tools: List[Any] = []
-        if read_csvs:
+        if all or enable_read_csv_file:
             tools.append(self.read_csv_file)
-        if list_csvs:
+        if all or enable_list_csv_files:
             tools.append(self.list_csv_files)
-        if read_column_names:
+        if all or enable_get_columns:
             tools.append(self.get_columns)
-        if query_csvs:
+        if all or enable_query_csv_file:
             try:
                 import duckdb  # noqa: F401
 
