@@ -44,6 +44,8 @@ def print_response_stream(
     console: Optional[Any] = None,
     add_history_to_context: Optional[bool] = None,
     dependencies: Optional[Dict[str, Any]] = None,
+    add_dependencies_to_context: Optional[bool] = None,
+    add_session_state_to_context: Optional[bool] = None,
     metadata: Optional[Dict[str, Any]] = None,
     **kwargs: Any,
 ):
@@ -76,6 +78,8 @@ def print_response_stream(
         if render:
             live_log.update(Group(*panels))
 
+        input_content = get_text_from_message(input)
+
         for response_event in agent.run(
             input=input,
             session_id=session_id,
@@ -90,6 +94,8 @@ def print_response_stream(
             knowledge_filters=knowledge_filters,
             debug_mode=debug_mode,
             add_history_to_context=add_history_to_context,
+            add_dependencies_to_context=add_dependencies_to_context,
+            add_session_state_to_context=add_session_state_to_context,
             dependencies=dependencies,
             metadata=metadata,
             **kwargs,
@@ -102,6 +108,10 @@ def print_response_stream(
                     live_log.update(Group(*panels))
                     return
 
+                if response_event.event == RunEvent.pre_hook_completed:  # type: ignore
+                    if response_event.run_input is not None:  # type: ignore
+                        input_content = get_text_from_message(response_event.run_input.input_content)  # type: ignore
+
                 if (
                     response_event.event == RunEvent.tool_call_started  # type: ignore
                     and hasattr(response_event, "tool")
@@ -112,7 +122,9 @@ def print_response_stream(
                 if response_event.event == RunEvent.run_content:  # type: ignore
                     if hasattr(response_event, "content"):
                         if isinstance(response_event.content, str):
-                            _response_content += response_event.content
+                            # Don't accumulate text content, parser_model will replace it
+                            if not (agent.parser_model is not None and agent.output_schema is not None):
+                                _response_content += response_event.content
                         elif agent.output_schema is not None and isinstance(response_event.content, BaseModel):
                             try:
                                 response_content_batch = JSON(  # type: ignore
@@ -151,9 +163,8 @@ def print_response_stream(
             panels = [status]
             if show_message:
                 # Convert message to a panel
-                message_content = get_text_from_message(input)
                 message_panel = create_panel(
-                    content=Text(message_content, style="green"),
+                    content=Text(input_content, style="green"),
                     title="Message",
                     border_style="cyan",
                 )
@@ -221,6 +232,8 @@ async def aprint_response_stream(
     console: Optional[Any] = None,
     add_history_to_context: Optional[bool] = None,
     dependencies: Optional[Dict[str, Any]] = None,
+    add_dependencies_to_context: Optional[bool] = None,
+    add_session_state_to_context: Optional[bool] = None,
     metadata: Optional[Dict[str, Any]] = None,
     **kwargs: Any,
 ):
@@ -267,11 +280,15 @@ async def aprint_response_stream(
             knowledge_filters=knowledge_filters,
             debug_mode=debug_mode,
             add_history_to_context=add_history_to_context,
+            add_dependencies_to_context=add_dependencies_to_context,
+            add_session_state_to_context=add_session_state_to_context,
             dependencies=dependencies,
             metadata=metadata,
             **kwargs,
         )
 
+        input_content = get_text_from_message(input)
+
         async for resp in result:  # type: ignore
             if isinstance(resp, tuple(get_args(RunOutputEvent))):
                 if resp.is_paused:
@@ -287,9 +304,15 @@ async def aprint_response_stream(
                 ):
                     accumulated_tool_calls.append(resp.tool)
 
+                if resp.event == RunEvent.pre_hook_completed:  # type: ignore
+                    if resp.run_input is not None:  # type: ignore
+                        input_content = get_text_from_message(resp.run_input.input_content)  # type: ignore
+
                 if resp.event == RunEvent.run_content:  # type: ignore
                     if isinstance(resp.content, str):
-                        _response_content += resp.content
+                        # Don't accumulate text content, parser_model will replace it
+                        if not (agent.parser_model is not None and agent.output_schema is not None):
+                            _response_content += resp.content
                     elif agent.output_schema is not None and isinstance(resp.content, BaseModel):
                         try:
                             response_content_batch = JSON(resp.content.model_dump_json(exclude_none=True), indent=2)  # type: ignore
@@ -326,12 +349,11 @@ async def aprint_response_stream(
 
             panels = [status]
 
-            if input and show_message:
+            if input_content and show_message:
                 render = True
                 # Convert message to a panel
-                message_content = get_text_from_message(input)
                 message_panel = create_panel(
-                    content=Text(message_content, style="green"),
+                    content=Text(input_content, style="green"),
                     title="Message",
                     border_style="cyan",
                 )
@@ -411,7 +433,7 @@ def build_panels_stream(
             reasoning_panel = create_panel(content=step_content, title=f"Reasoning step {i}", border_style="green")
             panels.append(reasoning_panel)
 
-    if len(response_reasoning_content_buffer) > 0:
+    if len(response_reasoning_content_buffer) > 0 and show_reasoning:
         # Create panel for thinking
         thinking_panel = create_panel(
             content=Text(response_reasoning_content_buffer),
@@ -486,6 +508,8 @@ def print_response(
     console: Optional[Any] = None,
     add_history_to_context: Optional[bool] = None,
     dependencies: Optional[Dict[str, Any]] = None,
+    add_dependencies_to_context: Optional[bool] = None,
+    add_session_state_to_context: Optional[bool] = None,
     metadata: Optional[Dict[str, Any]] = None,
     **kwargs: Any,
 ):
@@ -523,12 +547,28 @@ def print_response(
             knowledge_filters=knowledge_filters,
             debug_mode=debug_mode,
             add_history_to_context=add_history_to_context,
+            add_dependencies_to_context=add_dependencies_to_context,
+            add_session_state_to_context=add_session_state_to_context,
             dependencies=dependencies,
             metadata=metadata,
             **kwargs,
         )
         response_timer.stop()
 
+        if run_response.input is not None and run_response.input.input_content != input:
+            # Input was modified during the run
+            panels = [status]
+            if show_message:
+                # Convert message to a panel
+                message_content = get_text_from_message(run_response.input.input_content)
+                message_panel = create_panel(
+                    content=Text(message_content, style="green"),
+                    title="Message",
+                    border_style="cyan",
+                )
+                panels.append(message_panel)  # type: ignore
+                live_log.update(Group(*panels))
+
         additional_panels = build_panels(
             run_response=run_response,
             output_schema=agent.output_schema,  # type: ignore
@@ -586,6 +626,8 @@ async def aprint_response(
     console: Optional[Any] = None,
     add_history_to_context: Optional[bool] = None,
     dependencies: Optional[Dict[str, Any]] = None,
+    add_dependencies_to_context: Optional[bool] = None,
+    add_session_state_to_context: Optional[bool] = None,
     metadata: Optional[Dict[str, Any]] = None,
     **kwargs: Any,
 ):
@@ -623,12 +665,28 @@ async def aprint_response(
             knowledge_filters=knowledge_filters,
             debug_mode=debug_mode,
             add_history_to_context=add_history_to_context,
+            add_dependencies_to_context=add_dependencies_to_context,
+            add_session_state_to_context=add_session_state_to_context,
             dependencies=dependencies,
             metadata=metadata,
             **kwargs,
         )
         response_timer.stop()
 
+        if run_response.input is not None and run_response.input.input_content != input:
+            # Input was modified during the run
+            panels = [status]
+            if show_message:
+                # Convert message to a panel
+                message_content = get_text_from_message(run_response.input.input_content)
+                message_panel = create_panel(
+                    content=Text(message_content, style="green"),
+                    title="Message",
+                    border_style="cyan",
+                )
+                panels.append(message_panel)  # type: ignore
+                live_log.update(Group(*panels))
+
         additional_panels = build_panels(
             run_response=run_response,
             output_schema=agent.output_schema,  # type: ignore
@@ -707,7 +765,7 @@ def build_panels(
             reasoning_panel = create_panel(content=step_content, title=f"Reasoning step {i}", border_style="green")
             panels.append(reasoning_panel)
 
-    if isinstance(run_response, RunOutput) and run_response.reasoning_content is not None:
+    if isinstance(run_response, RunOutput) and run_response.reasoning_content is not None and show_reasoning:
         # Create panel for thinking
         thinking_panel = create_panel(
             content=Text(run_response.reasoning_content),
