@@ -36,6 +36,8 @@ def print_response(
     knowledge_filters: Optional[Dict[str, Any]] = None,
     add_history_to_context: Optional[bool] = None,
     dependencies: Optional[Dict[str, Any]] = None,
+    add_dependencies_to_context: Optional[bool] = None,
+    add_session_state_to_context: Optional[bool] = None,
     metadata: Optional[Dict[str, Any]] = None,
     debug_mode: Optional[bool] = None,
     **kwargs: Any,
@@ -88,12 +90,28 @@ def print_response(
             knowledge_filters=knowledge_filters,
             add_history_to_context=add_history_to_context,
             dependencies=dependencies,
+            add_dependencies_to_context=add_dependencies_to_context,
+            add_session_state_to_context=add_session_state_to_context,
             metadata=metadata,
             debug_mode=debug_mode,
             **kwargs,
         )
         response_timer.stop()
 
+        if run_response.input is not None and run_response.input.input_content != input:
+            # Input was modified during the run
+            panels = [status]
+            if show_message:
+                # Convert message to a panel
+                message_content = get_text_from_message(run_response.input.input_content)
+                message_panel = create_panel(
+                    content=Text(message_content, style="green"),
+                    title="Message",
+                    border_style="cyan",
+                )
+                panels.append(message_panel)  # type: ignore
+                live_console.update(Group(*panels))
+
         team_markdown = False
         member_markdown = {}
         if markdown:
@@ -121,7 +139,7 @@ def print_response(
                 panels.append(reasoning_panel)
             live_console.update(Group(*panels))
 
-        if isinstance(run_response, TeamRunOutput) and run_response.reasoning_content is not None:
+        if isinstance(run_response, TeamRunOutput) and run_response.reasoning_content is not None and show_reasoning:
             # Create panel for thinking
             thinking_panel = create_panel(
                 content=Text(run_response.reasoning_content),
@@ -317,6 +335,8 @@ def print_response_stream(
     knowledge_filters: Optional[Dict[str, Any]] = None,
     add_history_to_context: Optional[bool] = None,
     dependencies: Optional[Dict[str, Any]] = None,
+    add_dependencies_to_context: Optional[bool] = None,
+    add_session_state_to_context: Optional[bool] = None,
     metadata: Optional[Dict[str, Any]] = None,
     debug_mode: Optional[bool] = None,
     **kwargs: Any,
@@ -386,12 +406,16 @@ def print_response_stream(
             knowledge_filters=knowledge_filters,
             add_history_to_context=add_history_to_context,
             dependencies=dependencies,
+            add_dependencies_to_context=add_dependencies_to_context,
+            add_session_state_to_context=add_session_state_to_context,
             metadata=metadata,
             debug_mode=debug_mode,
             yield_run_response=True,
             **kwargs,
         )
 
+        input_content = get_text_from_message(input)
+
         team_markdown = None
         member_markdown = {}
 
@@ -427,6 +451,10 @@ def print_response_stream(
                 if hasattr(resp, "reasoning_steps") and resp.reasoning_steps is not None:  # type: ignore
                     reasoning_steps = resp.reasoning_steps  # type: ignore
 
+                if resp.event == TeamRunEvent.pre_hook_completed:  # type: ignore
+                    if resp.run_input is not None:  # type: ignore
+                        input_content = get_text_from_message(resp.run_input.input_content)  # type: ignore
+
                 # Collect team tool calls, avoiding duplicates
                 if resp.event == TeamRunEvent.tool_call_completed and resp.tool:  # type: ignore
                     tool = resp.tool  # type: ignore
@@ -471,12 +499,11 @@ def print_response_stream(
             # Create new panels for each chunk
             panels = []
 
-            if input and show_message:
+            if input_content and show_message:
                 render = True
                 # Convert message to a panel
-                message_content = get_text_from_message(input)
                 message_panel = create_panel(
-                    content=Text(message_content, style="green"),
+                    content=Text(input_content, style="green"),
                     title="Message",
                     border_style="cyan",
                 )
@@ -489,7 +516,7 @@ def print_response_stream(
                     reasoning_panel = build_reasoning_step_panel(i, step, show_full_reasoning)
                     panels.append(reasoning_panel)
 
-            if len(_response_reasoning_content) > 0:
+            if len(_response_reasoning_content) > 0 and show_reasoning:
                 render = True
                 # Create panel for thinking
                 thinking_panel = create_panel(
@@ -655,10 +682,9 @@ def print_response_stream(
         final_panels = []
 
         # Start with the message
-        if input and show_message:
-            message_content = get_text_from_message(input)
+        if input_content and show_message:
             message_panel = create_panel(
-                content=Text(message_content, style="green"),
+                content=Text(input_content, style="green"),
                 title="Message",
                 border_style="cyan",
             )
@@ -671,7 +697,7 @@ def print_response_stream(
                 final_panels.append(reasoning_panel)
 
         # Add thinking panel if available
-        if _response_reasoning_content:
+        if _response_reasoning_content and show_reasoning:
             thinking_panel = create_panel(
                 content=Text(_response_reasoning_content),
                 title=f"Thinking ({response_timer.elapsed:.1f}s)",
@@ -841,6 +867,8 @@ async def aprint_response(
     knowledge_filters: Optional[Dict[str, Any]] = None,
     add_history_to_context: Optional[bool] = None,
     dependencies: Optional[Dict[str, Any]] = None,
+    add_dependencies_to_context: Optional[bool] = None,
+    add_session_state_to_context: Optional[bool] = None,
     metadata: Optional[Dict[str, Any]] = None,
     debug_mode: Optional[bool] = None,
     **kwargs: Any,
@@ -893,12 +921,28 @@ async def aprint_response(
             knowledge_filters=knowledge_filters,
             add_history_to_context=add_history_to_context,
             dependencies=dependencies,
+            add_dependencies_to_context=add_dependencies_to_context,
+            add_session_state_to_context=add_session_state_to_context,
             metadata=metadata,
             debug_mode=debug_mode,
             **kwargs,
         )
         response_timer.stop()
 
+        if run_response.input is not None and run_response.input.input_content != input:
+            # Input was modified during the run
+            panels = [status]
+            if show_message:
+                # Convert message to a panel
+                message_content = get_text_from_message(run_response.input.input_content)
+                message_panel = create_panel(
+                    content=Text(message_content, style="green"),
+                    title="Message",
+                    border_style="cyan",
+                )
+                panels.append(message_panel)  # type: ignore
+                live_console.update(Group(*panels))
+
         team_markdown = False
         member_markdown = {}
         if markdown:
@@ -926,7 +970,7 @@ async def aprint_response(
                 panels.append(reasoning_panel)
             live_console.update(Group(*panels))
 
-        if isinstance(run_response, TeamRunOutput) and run_response.reasoning_content is not None:
+        if isinstance(run_response, TeamRunOutput) and run_response.reasoning_content is not None and show_reasoning:
             # Create panel for thinking
             thinking_panel = create_panel(
                 content=Text(run_response.reasoning_content),
@@ -1120,6 +1164,8 @@ async def aprint_response_stream(
     knowledge_filters: Optional[Dict[str, Any]] = None,
     add_history_to_context: Optional[bool] = None,
     dependencies: Optional[Dict[str, Any]] = None,
+    add_dependencies_to_context: Optional[bool] = None,
+    add_session_state_to_context: Optional[bool] = None,
     metadata: Optional[Dict[str, Any]] = None,
     debug_mode: Optional[bool] = None,
     **kwargs: Any,
@@ -1182,6 +1228,8 @@ async def aprint_response_stream(
         # Dict to track member response panels by member_id
         member_response_panels = {}
 
+        input_content = get_text_from_message(input)
+
         final_run_response = None
         async for resp in team.arun(  # type: ignore
             input=input,
@@ -1196,6 +1244,8 @@ async def aprint_response_stream(
             user_id=user_id,
             knowledge_filters=knowledge_filters,
             add_history_to_context=add_history_to_context,
+            add_dependencies_to_context=add_dependencies_to_context,
+            add_session_state_to_context=add_session_state_to_context,
             dependencies=dependencies,
             metadata=metadata,
             debug_mode=debug_mode,
@@ -1229,6 +1279,10 @@ async def aprint_response_stream(
                 if hasattr(resp, "reasoning_steps") and resp.reasoning_steps is not None:  # type: ignore
                     reasoning_steps = resp.reasoning_steps  # type: ignore
 
+                if resp.event == TeamRunEvent.pre_hook_completed:  # type: ignore
+                    if resp.run_input is not None:  # type: ignore
+                        input_content = get_text_from_message(resp.run_input.input_content)  # type: ignore
+
                 # Collect team tool calls, avoiding duplicates
                 if resp.event == TeamRunEvent.tool_call_completed and resp.tool:  # type: ignore
                     tool = resp.tool  # type: ignore
@@ -1272,12 +1326,11 @@ async def aprint_response_stream(
             # Create new panels for each chunk
             panels = []
 
-            if input and show_message:
+            if input_content and show_message:
                 render = True
                 # Convert message to a panel
-                message_content = get_text_from_message(input)
                 message_panel = create_panel(
-                    content=Text(message_content, style="green"),
+                    content=Text(input_content, style="green"),
                     title="Message",
                     border_style="cyan",
                 )
@@ -1290,7 +1343,7 @@ async def aprint_response_stream(
                     reasoning_panel = build_reasoning_step_panel(i, step, show_full_reasoning)
                     panels.append(reasoning_panel)
 
-            if len(_response_reasoning_content) > 0:
+            if len(_response_reasoning_content) > 0 and show_reasoning:
                 render = True
                 # Create panel for thinking
                 thinking_panel = create_panel(
@@ -1457,10 +1510,9 @@ async def aprint_response_stream(
         final_panels = []
 
         # Start with the message
-        if input and show_message:
-            message_content = get_text_from_message(input)
+        if input_content and show_message:
             message_panel = create_panel(
-                content=Text(message_content, style="green"),
+                content=Text(input_content, style="green"),
                 title="Message",
                 border_style="cyan",
             )
@@ -1473,7 +1525,7 @@ async def aprint_response_stream(
                 final_panels.append(reasoning_panel)
 
         # Add thinking panel if available
-        if _response_reasoning_content:
+        if _response_reasoning_content and show_reasoning:
             thinking_panel = create_panel(
                 content=Text(_response_reasoning_content),
                 title=f"Thinking ({response_timer.elapsed:.1f}s)",
