@@ -1,3 +1,9 @@
 from agno.vectordb.clickhouse.clickhousedb import Clickhouse
 from agno.vectordb.clickhouse.index import HNSW
 from agno.vectordb.distance import Distance
+
+__all__ = [
+    "Clickhouse",
+    "HNSW",
+    "Distance",
+]
