@@ -1,5 +1,3 @@
 from agno.vectordb.mongodb.mongodb import MongoDb
 
-__all__ = [
-    "MongoDb",
-]
+__all__ = ["MongoDb"]
