@@ -11,6 +11,7 @@ from agno.run.workflow import (
     WorkflowRunOutput,
     WorkflowRunOutputEvent,
 )
+from agno.session.workflow import WorkflowSession
 from agno.utils.log import log_debug, logger
 from agno.workflow.step import Step
 from agno.workflow.types import StepInput, StepOutput, StepType
@@ -110,30 +111,14 @@ class Condition:
             audio=current_audio + all_audio,
         )
 
-    def _evaluate_condition(self, step_input: StepInput) -> bool:
+    def _evaluate_condition(self, step_input: StepInput, session_state: Optional[Dict[str, Any]] = None) -> bool:
         """Evaluate the condition and return boolean result"""
         if isinstance(self.evaluator, bool):
             return self.evaluator
 
         if callable(self.evaluator):
-            result = self.evaluator(step_input)
-
-            if isinstance(result, bool):
-                return result
-            else:
-                logger.warning(f"Condition evaluator returned unexpected type: {type(result)}, expected bool")
-                return False
-
-        return False
-
-    async def _aevaluate_condition(self, step_input: StepInput) -> bool:
-        """Async version of condition evaluation"""
-        if isinstance(self.evaluator, bool):
-            return self.evaluator
-
-        if callable(self.evaluator):
-            if inspect.iscoroutinefunction(self.evaluator):
-                result = await self.evaluator(step_input)
+            if session_state is not None and self._evaluator_has_session_state_param():
+                result = self.evaluator(step_input, session_state=session_state)  # type: ignore[call-arg]
             else:
                 result = self.evaluator(step_input)
 
@@ -145,6 +130,44 @@ class Condition:
 
         return False
 
+    async def _aevaluate_condition(self, step_input: StepInput, session_state: Optional[Dict[str, Any]] = None) -> bool:
+        """Async version of condition evaluation"""
+        if isinstance(self.evaluator, bool):
+            return self.evaluator
+
+        if callable(self.evaluator):
+            has_session_state = session_state is not None and self._evaluator_has_session_state_param()
+
+            if inspect.iscoroutinefunction(self.evaluator):
+                if has_session_state:
+                    result = await self.evaluator(step_input, session_state=session_state)  # type: ignore[call-arg]
+                else:
+                    result = await self.evaluator(step_input)
+            else:
+                if has_session_state:
+                    result = self.evaluator(step_input, session_state=session_state)  # type: ignore[call-arg]
+                else:
+                    result = self.evaluator(step_input)
+
+            if isinstance(result, bool):
+                return result
+            else:
+                logger.warning(f"Condition evaluator returned unexpected type: {type(result)}, expected bool")
+                return False
+
+        return False
+
+    def _evaluator_has_session_state_param(self) -> bool:
+        """Check if the evaluator function has a session_state parameter"""
+        if not callable(self.evaluator):
+            return False
+
+        try:
+            sig = inspect.signature(self.evaluator)
+            return "session_state" in sig.parameters
+        except Exception:
+            return False
+
     def execute(
         self,
         step_input: StepInput,
@@ -153,6 +176,9 @@ class Condition:
         workflow_run_response: Optional[WorkflowRunOutput] = None,
         store_executor_outputs: bool = True,
         session_state: Optional[Dict[str, Any]] = None,
+        workflow_session: Optional[WorkflowSession] = None,
+        add_workflow_history_to_steps: Optional[bool] = False,
+        num_history_runs: int = 3,
     ) -> StepOutput:
         """Execute the condition and its steps with sequential chaining if condition is true"""
         log_debug(f"Condition Start: {self.name}", center=True, symbol="-")
@@ -162,7 +188,7 @@ class Condition:
         self._prepare_steps()
 
         # Evaluate the condition
-        condition_result = self._evaluate_condition(step_input)
+        condition_result = self._evaluate_condition(step_input, session_state)
         log_debug(f"Condition {self.name} evaluated to: {condition_result}")
 
         if not condition_result:
@@ -189,6 +215,9 @@ class Condition:
                     workflow_run_response=workflow_run_response,
                     store_executor_outputs=store_executor_outputs,
                     session_state=session_state,
+                    workflow_session=workflow_session,
+                    add_workflow_history_to_steps=add_workflow_history_to_steps,
+                    num_history_runs=num_history_runs,
                 )
 
                 # Handle both single StepOutput and List[StepOutput] (from Loop/Condition/Router steps)
@@ -250,11 +279,15 @@ class Condition:
         session_id: Optional[str] = None,
         user_id: Optional[str] = None,
         stream_intermediate_steps: bool = False,
+        stream_executor_events: bool = True,
         workflow_run_response: Optional[WorkflowRunOutput] = None,
         step_index: Optional[Union[int, tuple]] = None,
         store_executor_outputs: bool = True,
         session_state: Optional[Dict[str, Any]] = None,
         parent_step_id: Optional[str] = None,
+        workflow_session: Optional[WorkflowSession] = None,
+        add_workflow_history_to_steps: Optional[bool] = False,
+        num_history_runs: int = 3,
     ) -> Iterator[Union[WorkflowRunOutputEvent, StepOutput]]:
         """Execute the condition with streaming support - mirrors Loop logic"""
         log_debug(f"Condition Start: {self.name}", center=True, symbol="-")
@@ -264,7 +297,7 @@ class Condition:
         self._prepare_steps()
 
         # Evaluate the condition
-        condition_result = self._evaluate_condition(step_input)
+        condition_result = self._evaluate_condition(step_input, session_state)
         log_debug(f"Condition {self.name} evaluated to: {condition_result}")
 
         if stream_intermediate_steps and workflow_run_response:
@@ -322,11 +355,15 @@ class Condition:
                     session_id=session_id,
                     user_id=user_id,
                     stream_intermediate_steps=stream_intermediate_steps,
+                    stream_executor_events=stream_executor_events,
                     workflow_run_response=workflow_run_response,
                     step_index=child_step_index,
                     store_executor_outputs=store_executor_outputs,
                     session_state=session_state,
                     parent_step_id=conditional_step_id,
+                    workflow_session=workflow_session,
+                    add_workflow_history_to_steps=add_workflow_history_to_steps,
+                    num_history_runs=num_history_runs,
                 ):
                     if isinstance(event, StepOutput):
                         step_outputs_for_step.append(event)
@@ -407,6 +444,9 @@ class Condition:
         workflow_run_response: Optional[WorkflowRunOutput] = None,
         store_executor_outputs: bool = True,
         session_state: Optional[Dict[str, Any]] = None,
+        workflow_session: Optional[WorkflowSession] = None,
+        add_workflow_history_to_steps: Optional[bool] = False,
+        num_history_runs: int = 3,
     ) -> StepOutput:
         """Async execute the condition and its steps with sequential chaining"""
         log_debug(f"Condition Start: {self.name}", center=True, symbol="-")
@@ -416,7 +456,7 @@ class Condition:
         self._prepare_steps()
 
         # Evaluate the condition
-        condition_result = await self._aevaluate_condition(step_input)
+        condition_result = await self._aevaluate_condition(step_input, session_state)
         log_debug(f"Condition {self.name} evaluated to: {condition_result}")
 
         if not condition_result:
@@ -445,6 +485,9 @@ class Condition:
                     workflow_run_response=workflow_run_response,
                     store_executor_outputs=store_executor_outputs,
                     session_state=session_state,
+                    workflow_session=workflow_session,
+                    add_workflow_history_to_steps=add_workflow_history_to_steps,
+                    num_history_runs=num_history_runs,
                 )
 
                 # Handle both single StepOutput and List[StepOutput]
@@ -504,11 +547,15 @@ class Condition:
         session_id: Optional[str] = None,
         user_id: Optional[str] = None,
         stream_intermediate_steps: bool = False,
+        stream_executor_events: bool = True,
         workflow_run_response: Optional[WorkflowRunOutput] = None,
         step_index: Optional[Union[int, tuple]] = None,
         store_executor_outputs: bool = True,
         session_state: Optional[Dict[str, Any]] = None,
         parent_step_id: Optional[str] = None,
+        workflow_session: Optional[WorkflowSession] = None,
+        add_workflow_history_to_steps: Optional[bool] = False,
+        num_history_runs: int = 3,
     ) -> AsyncIterator[Union[WorkflowRunOutputEvent, TeamRunOutputEvent, RunOutputEvent, StepOutput]]:
         """Async execute the condition with streaming support - mirrors Loop logic"""
         log_debug(f"Condition Start: {self.name}", center=True, symbol="-")
@@ -518,7 +565,7 @@ class Condition:
         self._prepare_steps()
 
         # Evaluate the condition
-        condition_result = await self._aevaluate_condition(step_input)
+        condition_result = await self._aevaluate_condition(step_input, session_state)
         log_debug(f"Condition {self.name} evaluated to: {condition_result}")
 
         if stream_intermediate_steps and workflow_run_response:
@@ -578,11 +625,15 @@ class Condition:
                     session_id=session_id,
                     user_id=user_id,
                     stream_intermediate_steps=stream_intermediate_steps,
+                    stream_executor_events=stream_executor_events,
                     workflow_run_response=workflow_run_response,
                     step_index=child_step_index,
                     store_executor_outputs=store_executor_outputs,
                     session_state=session_state,
                     parent_step_id=conditional_step_id,
+                    workflow_session=workflow_session,
+                    add_workflow_history_to_steps=add_workflow_history_to_steps,
+                    num_history_runs=num_history_runs,
                 ):
                     if isinstance(event, StepOutput):
                         step_outputs_for_step.append(event)
