@@ -5,6 +5,7 @@ from typing import Any, AsyncIterator, Awaitable, Callable, Dict, Iterator, List
 from uuid import uuid4
 
 from pydantic import BaseModel
+from typing_extensions import TypeGuard
 
 from agno.agent import Agent
 from agno.media import Audio, Image, Video
@@ -17,6 +18,7 @@ from agno.run.workflow import (
     WorkflowRunOutput,
     WorkflowRunOutputEvent,
 )
+from agno.session.workflow import WorkflowSession
 from agno.team import Team
 from agno.utils.log import log_debug, logger, use_agent_logger, use_team_logger, use_workflow_logger
 from agno.utils.merge_dict import merge_dictionaries
@@ -60,6 +62,9 @@ class Step:
     # If False, only warn about missing inputs
     strict_input_validation: bool = False
 
+    add_workflow_history: Optional[bool] = None
+    num_history_runs: int = 3
+
     _retry_count: int = 0
 
     def __init__(
@@ -74,6 +79,8 @@ class Step:
         timeout_seconds: Optional[int] = None,
         skip_on_failure: bool = False,
         strict_input_validation: bool = False,
+        add_workflow_history: Optional[bool] = None,
+        num_history_runs: int = 3,
     ):
         # Auto-detect name for function executors if not provided
         if name is None and executor is not None:
@@ -93,6 +100,8 @@ class Step:
         self.timeout_seconds = timeout_seconds
         self.skip_on_failure = skip_on_failure
         self.strict_input_validation = strict_input_validation
+        self.add_workflow_history = add_workflow_history
+        self.num_history_runs = num_history_runs
         self.step_id = step_id
 
         if step_id is None:
@@ -164,6 +173,37 @@ class Step:
             return response.metrics
         return None
 
+    def _call_custom_function(
+        self,
+        func: Callable,
+        step_input: StepInput,
+        session_state: Optional[Dict[str, Any]] = None,
+    ) -> Any:
+        """Call custom function with session_state support if the function accepts it"""
+        if session_state is not None and self._function_has_session_state_param():
+            return func(step_input, session_state)
+        else:
+            return func(step_input)
+
+    async def _acall_custom_function(
+        self,
+        func: Callable,
+        step_input: StepInput,
+        session_state: Optional[Dict[str, Any]] = None,
+    ) -> Any:
+        """Call custom async function with session_state support if the function accepts it"""
+
+        if _is_async_generator_function(func):
+            if session_state is not None and self._function_has_session_state_param():
+                return func(step_input, session_state)
+            else:
+                return func(step_input)
+        else:
+            if session_state is not None and self._function_has_session_state_param():
+                return await func(step_input, session_state)
+            else:
+                return await func(step_input)
+
     def execute(
         self,
         step_input: StepInput,
@@ -172,6 +212,9 @@ class Step:
         workflow_run_response: Optional["WorkflowRunOutput"] = None,
         session_state: Optional[Dict[str, Any]] = None,
         store_executor_outputs: bool = True,
+        workflow_session: Optional[WorkflowSession] = None,
+        add_workflow_history_to_steps: Optional[bool] = False,
+        num_history_runs: int = 3,
     ) -> StepOutput:
         """Execute the step with StepInput, returning final StepOutput (non-streaming)"""
         log_debug(f"Executing step: {self.name}")
@@ -179,20 +222,26 @@ class Step:
         if step_input.previous_step_outputs:
             step_input.previous_step_content = step_input.get_last_step_content()
 
+        if workflow_session:
+            step_input.workflow_session = workflow_session
+        session_state_copy = copy(session_state) if session_state is not None else {}
+
         # Execute with retries
         for attempt in range(self.max_retries + 1):
             try:
                 response: Union[RunOutput, TeamRunOutput, StepOutput]
                 if self._executor_type == "function":
-                    if inspect.iscoroutinefunction(self.active_executor) or inspect.isasyncgenfunction(
-                        self.active_executor
-                    ):
+                    if _is_async_callable(self.active_executor) or _is_async_generator_function(self.active_executor):
                         raise ValueError("Cannot use async function with synchronous execution")
-                    if inspect.isgeneratorfunction(self.active_executor):
+                    if _is_generator_function(self.active_executor):
                         content = ""
                         final_response = None
                         try:
-                            for chunk in self.active_executor(step_input):  # type: ignore
+                            for chunk in self._call_custom_function(
+                                self.active_executor,
+                                step_input,
+                                session_state_copy,  # type: ignore[arg-type]
+                            ):  # type: ignore
                                 if (
                                     hasattr(chunk, "content")
                                     and chunk.content is not None
@@ -208,13 +257,21 @@ class Step:
                             if hasattr(e, "value") and isinstance(e.value, StepOutput):
                                 final_response = e.value
 
+                        # Merge session_state changes back
+                        if session_state is not None:
+                            merge_dictionaries(session_state, session_state_copy)
+
                         if final_response is not None:
                             response = final_response
                         else:
                             response = StepOutput(content=content)
                     else:
-                        # Execute function directly with StepInput
-                        result = self.active_executor(step_input)  # type: ignore
+                        # Execute function with signature inspection for session_state support
+                        result = self._call_custom_function(self.active_executor, step_input, session_state_copy)  # type: ignore
+
+                        # Merge session_state changes back
+                        if session_state is not None:
+                            merge_dictionaries(session_state, session_state_copy)
 
                         # If function returns StepOutput, use it directly
                         if isinstance(result, StepOutput):
@@ -248,9 +305,22 @@ class Step:
                         if isinstance(self.active_executor, Team):
                             kwargs["store_member_responses"] = True
 
-                        session_state_copy = copy(session_state)
+                        num_history_runs = self.num_history_runs if self.num_history_runs else num_history_runs
+
+                        use_history = (
+                            self.add_workflow_history
+                            if self.add_workflow_history is not None
+                            else add_workflow_history_to_steps
+                        )
+
+                        final_message = message
+                        if use_history and workflow_session:
+                            history_messages = workflow_session.get_workflow_history_context(num_runs=num_history_runs)
+                            if history_messages:
+                                final_message = f"{history_messages}{message}"
+
                         response = self.active_executor.run(  # type: ignore
-                            input=message,  # type: ignore
+                            input=final_message,  # type: ignore
                             images=images,
                             videos=videos,
                             audio=audios,
@@ -261,8 +331,9 @@ class Step:
                             **kwargs,
                         )
 
-                        # Update workflow session state
-                        merge_dictionaries(session_state, session_state_copy)  # type: ignore
+                        if session_state is not None:
+                            # Update workflow session state
+                            merge_dictionaries(session_state, session_state_copy)  # type: ignore
 
                         if store_executor_outputs and workflow_run_response is not None:
                             self._store_executor_response(workflow_run_response, response)  # type: ignore
@@ -291,23 +362,69 @@ class Step:
 
         return StepOutput(content=f"Step {self.name} failed but skipped", success=False)
 
+    def _function_has_session_state_param(self) -> bool:
+        """Check if the custom function has a session_state parameter"""
+        if self._executor_type != "function":
+            return False
+
+        try:
+            sig = inspect.signature(self.active_executor)  # type: ignore
+            return "session_state" in sig.parameters
+        except Exception:
+            return False
+
+    def _enrich_event_with_context(
+        self,
+        event: Any,
+        workflow_run_response: Optional["WorkflowRunOutput"] = None,
+        step_index: Optional[Union[int, tuple]] = None,
+    ) -> Any:
+        """Enrich event with step and workflow context information"""
+        if workflow_run_response is None:
+            return event
+
+        if hasattr(event, "workflow_id"):
+            event.workflow_id = workflow_run_response.workflow_id
+        if hasattr(event, "workflow_run_id"):
+            event.workflow_run_id = workflow_run_response.run_id
+        if hasattr(event, "step_id"):
+            event.step_id = self.step_id
+        if hasattr(event, "step_name") and self.name is not None:
+            if getattr(event, "step_name", None) is None:
+                event.step_name = self.name
+        # Only set step_index if it's not already set (preserve parallel.py's tuples)
+        if hasattr(event, "step_index") and step_index is not None:
+            if event.step_index is None:
+                event.step_index = step_index
+
+        return event
+
     def execute_stream(
         self,
         step_input: StepInput,
         session_id: Optional[str] = None,
         user_id: Optional[str] = None,
         stream_intermediate_steps: bool = False,
+        stream_executor_events: bool = True,
         workflow_run_response: Optional["WorkflowRunOutput"] = None,
         session_state: Optional[Dict[str, Any]] = None,
         step_index: Optional[Union[int, tuple]] = None,
         store_executor_outputs: bool = True,
         parent_step_id: Optional[str] = None,
+        workflow_session: Optional["WorkflowSession"] = None,
+        add_workflow_history_to_steps: Optional[bool] = False,
+        num_history_runs: int = 3,
     ) -> Iterator[Union[WorkflowRunOutputEvent, StepOutput]]:
         """Execute the step with event-driven streaming support"""
 
         if step_input.previous_step_outputs:
             step_input.previous_step_content = step_input.get_last_step_content()
 
+        if workflow_session:
+            step_input.workflow_session = workflow_session
+        # Create session_state copy once to avoid duplication
+        session_state_copy = copy(session_state) if session_state is not None else {}
+
         # Emit StepStartedEvent
         if stream_intermediate_steps and workflow_run_response:
             yield StepStartedEvent(
@@ -330,16 +447,15 @@ class Step:
                 if self._executor_type == "function":
                     log_debug(f"Executing function executor for step: {self.name}")
 
-                    if inspect.iscoroutinefunction(self.active_executor) or inspect.isasyncgenfunction(
-                        self.active_executor
-                    ):
+                    if _is_async_callable(self.active_executor) or _is_async_generator_function(self.active_executor):
                         raise ValueError("Cannot use async function with synchronous execution")
 
-                    if inspect.isgeneratorfunction(self.active_executor):
+                    if _is_generator_function(self.active_executor):
                         log_debug("Function returned iterable, streaming events")
                         content = ""
                         try:
-                            for event in self.active_executor(step_input):  # type: ignore
+                            iterator = self._call_custom_function(self.active_executor, step_input, session_state_copy)  # type: ignore
+                            for event in iterator:  # type: ignore
                                 if (
                                     hasattr(event, "content")
                                     and event.content is not None
@@ -352,7 +468,18 @@ class Step:
                                     final_response = event
                                     break
                                 else:
-                                    yield event  # type: ignore[misc]
+                                    # Enrich event with workflow context before yielding
+                                    enriched_event = self._enrich_event_with_context(
+                                        event, workflow_run_response, step_index
+                                    )
+                                    # Only yield executor events if stream_executor_events is True
+                                    if stream_executor_events:
+                                        yield enriched_event  # type: ignore[misc]
+
+                            # Merge session_state changes back
+                            if session_state is not None:
+                                merge_dictionaries(session_state, session_state_copy)
+
                             if not final_response:
                                 final_response = StepOutput(content=content)
                         except StopIteration as e:
@@ -360,7 +487,12 @@ class Step:
                                 final_response = e.value
 
                     else:
-                        result = self.active_executor(step_input)  # type: ignore
+                        result = self._call_custom_function(self.active_executor, step_input, session_state_copy)  # type: ignore
+
+                        # Merge session_state changes back
+                        if session_state is not None:
+                            merge_dictionaries(session_state, session_state_copy)
+
                         if isinstance(result, StepOutput):
                             final_response = result
                         else:
@@ -392,9 +524,22 @@ class Step:
                         if isinstance(self.active_executor, Team):
                             kwargs["store_member_responses"] = True
 
-                        session_state_copy = copy(session_state)
+                        num_history_runs = self.num_history_runs if self.num_history_runs else num_history_runs
+
+                        use_history = (
+                            self.add_workflow_history
+                            if self.add_workflow_history is not None
+                            else add_workflow_history_to_steps
+                        )
+
+                        final_message = message
+                        if use_history and workflow_session:
+                            history_messages = workflow_session.get_workflow_history_context(num_runs=num_history_runs)
+                            if history_messages:
+                                final_message = f"{history_messages}{message}"
+
                         response_stream = self.active_executor.run(  # type: ignore[call-overload, misc]
-                            input=message,
+                            input=final_message,
                             images=images,
                             videos=videos,
                             audio=audios,
@@ -404,14 +549,6 @@ class Step:
                             session_state=session_state_copy,  # Send a copy to the executor
                             stream=True,
                             stream_intermediate_steps=stream_intermediate_steps,
-                            # Pass workflow context directly via kwargs
-                            workflow_context={
-                                "workflow_id": workflow_run_response.workflow_id if workflow_run_response else None,
-                                "workflow_run_id": workflow_run_response.run_id if workflow_run_response else None,
-                                "step_id": self.step_id,
-                                "step_name": self.name,
-                                "step_index": step_index,
-                            },
                             yield_run_response=True,
                             **kwargs,
                         )
@@ -421,15 +558,19 @@ class Step:
                             if isinstance(event, RunOutput) or isinstance(event, TeamRunOutput):
                                 active_executor_run_response = event
                                 break
-                            yield event  # type: ignore[misc]
+                            enriched_event = self._enrich_event_with_context(event, workflow_run_response, step_index)
+                            # Only yield executor events if stream_executor_events is True
+                            if stream_executor_events:
+                                yield enriched_event  # type: ignore[misc]
 
-                        # Update workflow session state
-                        merge_dictionaries(session_state, session_state_copy)  # type: ignore
+                        if session_state is not None:
+                            # Update workflow session state
+                            merge_dictionaries(session_state, session_state_copy)  # type: ignore
 
                         if store_executor_outputs and workflow_run_response is not None:
                             self._store_executor_response(workflow_run_response, active_executor_run_response)  # type: ignore
 
-                        final_response = self._process_step_output(active_executor_run_response)  # type: ignore
+                        final_response = active_executor_run_response  # type: ignore
 
                     else:
                         raise ValueError(f"Unsupported executor type: {self._executor_type}")
@@ -443,6 +584,7 @@ class Step:
                 use_workflow_logger()
 
                 # Yield the step output
+                final_response = self._process_step_output(final_response)
                 yield final_response
 
                 # Emit StepCompletedEvent
@@ -486,6 +628,9 @@ class Step:
         workflow_run_response: Optional["WorkflowRunOutput"] = None,
         session_state: Optional[Dict[str, Any]] = None,
         store_executor_outputs: bool = True,
+        workflow_session: Optional["WorkflowSession"] = None,
+        add_workflow_history_to_steps: Optional[bool] = False,
+        num_history_runs: int = 3,
     ) -> StepOutput:
         """Execute the step with StepInput, returning final StepOutput (non-streaming)"""
         logger.info(f"Executing async step (non-streaming): {self.name}")
@@ -494,20 +639,28 @@ class Step:
         if step_input.previous_step_outputs:
             step_input.previous_step_content = step_input.get_last_step_content()
 
+        if workflow_session:
+            step_input.workflow_session = workflow_session
+        # Create session_state copy once to avoid duplication
+        session_state_copy = copy(session_state) if session_state is not None else {}
+
         # Execute with retries
         for attempt in range(self.max_retries + 1):
             try:
                 if self._executor_type == "function":
-                    import inspect
-
-                    if inspect.isgeneratorfunction(self.active_executor) or inspect.isasyncgenfunction(
+                    if _is_generator_function(self.active_executor) or _is_async_generator_function(
                         self.active_executor
                     ):
                         content = ""
                         final_response = None
                         try:
-                            if inspect.isgeneratorfunction(self.active_executor):
-                                for chunk in self.active_executor(step_input):  # type: ignore
+                            if _is_generator_function(self.active_executor):
+                                iterator = self._call_custom_function(
+                                    self.active_executor,
+                                    step_input,
+                                    session_state_copy,  # type: ignore[arg-type]
+                                )  # type: ignore
+                                for chunk in iterator:  # type: ignore
                                     if (
                                         hasattr(chunk, "content")
                                         and chunk.content is not None
@@ -519,8 +672,13 @@ class Step:
                                     if isinstance(chunk, StepOutput):
                                         final_response = chunk
                             else:
-                                if inspect.isasyncgenfunction(self.active_executor):
-                                    async for chunk in self.active_executor(step_input):  # type: ignore
+                                if _is_async_generator_function(self.active_executor):
+                                    iterator = await self._acall_custom_function(
+                                        self.active_executor,
+                                        step_input,
+                                        session_state_copy,  # type: ignore[arg-type]
+                                    )  # type: ignore
+                                    async for chunk in iterator:  # type: ignore
                                         if (
                                             hasattr(chunk, "content")
                                             and chunk.content is not None
@@ -536,15 +694,25 @@ class Step:
                             if hasattr(e, "value") and isinstance(e.value, StepOutput):
                                 final_response = e.value
 
+                        # Merge session_state changes back
+                        if session_state is not None:
+                            merge_dictionaries(session_state, session_state_copy)
+
                         if final_response is not None:
                             response = final_response
                         else:
                             response = StepOutput(content=content)
                     else:
-                        if inspect.iscoroutinefunction(self.active_executor):
-                            result = await self.active_executor(step_input)  # type: ignore
+                        if _is_async_callable(self.active_executor):
+                            result = await self._acall_custom_function(
+                                self.active_executor, step_input, session_state_copy
+                            )  # type: ignore
                         else:
-                            result = self.active_executor(step_input)  # type: ignore
+                            result = self._call_custom_function(self.active_executor, step_input, session_state_copy)  # type: ignore
+
+                        # Merge session_state changes back
+                        if session_state is not None:
+                            merge_dictionaries(session_state, session_state_copy)
 
                         # If function returns StepOutput, use it directly
                         if isinstance(result, StepOutput):
@@ -579,9 +747,22 @@ class Step:
                         if isinstance(self.active_executor, Team):
                             kwargs["store_member_responses"] = True
 
-                        session_state_copy = copy(session_state)
+                        num_history_runs = self.num_history_runs if self.num_history_runs else num_history_runs
+
+                        use_history = (
+                            self.add_workflow_history
+                            if self.add_workflow_history is not None
+                            else add_workflow_history_to_steps
+                        )
+
+                        final_message = message
+                        if use_history and workflow_session:
+                            history_messages = workflow_session.get_workflow_history_context(num_runs=num_history_runs)
+                            if history_messages:
+                                final_message = f"{history_messages}{message}"
+
                         response = await self.active_executor.arun(  # type: ignore
-                            input=message,  # type: ignore
+                            input=final_message,  # type: ignore
                             images=images,
                             videos=videos,
                             audio=audios,
@@ -592,8 +773,9 @@ class Step:
                             **kwargs,
                         )
 
-                        # Update workflow session state
-                        merge_dictionaries(session_state, session_state_copy)  # type: ignore
+                        if session_state is not None:
+                            # Update workflow session state
+                            merge_dictionaries(session_state, session_state_copy)  # type: ignore
 
                         if store_executor_outputs and workflow_run_response is not None:
                             self._store_executor_response(workflow_run_response, response)  # type: ignore
@@ -628,17 +810,27 @@ class Step:
         session_id: Optional[str] = None,
         user_id: Optional[str] = None,
         stream_intermediate_steps: bool = False,
+        stream_executor_events: bool = True,
         workflow_run_response: Optional["WorkflowRunOutput"] = None,
         session_state: Optional[Dict[str, Any]] = None,
         step_index: Optional[Union[int, tuple]] = None,
         store_executor_outputs: bool = True,
         parent_step_id: Optional[str] = None,
+        workflow_session: Optional["WorkflowSession"] = None,
+        add_workflow_history_to_steps: Optional[bool] = False,
+        num_history_runs: int = 3,
     ) -> AsyncIterator[Union[WorkflowRunOutputEvent, StepOutput]]:
         """Execute the step with event-driven streaming support"""
 
         if step_input.previous_step_outputs:
             step_input.previous_step_content = step_input.get_last_step_content()
 
+        if workflow_session:
+            step_input.workflow_session = workflow_session
+
+        # Create session_state copy once to avoid duplication
+        session_state_copy = copy(session_state) if session_state is not None else {}
+
         if stream_intermediate_steps and workflow_run_response:
             # Emit StepStartedEvent
             yield StepStartedEvent(
@@ -660,13 +852,17 @@ class Step:
 
                 if self._executor_type == "function":
                     log_debug(f"Executing async function executor for step: {self.name}")
-                    import inspect
 
                     # Check if the function is an async generator
-                    if inspect.isasyncgenfunction(self.active_executor):
+                    if _is_async_generator_function(self.active_executor):
                         content = ""
                         # It's an async generator - iterate over it
-                        async for event in self.active_executor(step_input):  # type: ignore
+                        iterator = await self._acall_custom_function(
+                            self.active_executor,
+                            step_input,
+                            session_state_copy,  # type: ignore[arg-type]
+                        )  # type: ignore
+                        async for event in iterator:  # type: ignore
                             if (
                                 hasattr(event, "content")
                                 and event.content is not None
@@ -679,20 +875,27 @@ class Step:
                                 final_response = event
                                 break
                             else:
-                                yield event  # type: ignore[misc]
+                                # Enrich event with workflow context before yielding
+                                enriched_event = self._enrich_event_with_context(
+                                    event, workflow_run_response, step_index
+                                )
+                                # Only yield executor events if stream_executor_events is True
+                                if stream_executor_events:
+                                    yield enriched_event  # type: ignore[misc]
                         if not final_response:
                             final_response = StepOutput(content=content)
-                    elif inspect.iscoroutinefunction(self.active_executor):
+                    elif _is_async_callable(self.active_executor):
                         # It's a regular async function - await it
-                        result = await self.active_executor(step_input)  # type: ignore
+                        result = await self._acall_custom_function(self.active_executor, step_input, session_state_copy)  # type: ignore
                         if isinstance(result, StepOutput):
                             final_response = result
                         else:
                             final_response = StepOutput(content=str(result))
-                    elif inspect.isgeneratorfunction(self.active_executor):
+                    elif _is_generator_function(self.active_executor):
                         content = ""
                         # It's a regular generator function - iterate over it
-                        for event in self.active_executor(step_input):  # type: ignore
+                        iterator = self._call_custom_function(self.active_executor, step_input, session_state_copy)  # type: ignore
+                        for event in iterator:  # type: ignore
                             if (
                                 hasattr(event, "content")
                                 and event.content is not None
@@ -705,16 +908,26 @@ class Step:
                                 final_response = event
                                 break
                             else:
-                                yield event  # type: ignore[misc]
+                                # Enrich event with workflow context before yielding
+                                enriched_event = self._enrich_event_with_context(
+                                    event, workflow_run_response, step_index
+                                )
+                                # Only yield executor events if stream_executor_events is True
+                                if stream_executor_events:
+                                    yield enriched_event  # type: ignore[misc]
                         if not final_response:
                             final_response = StepOutput(content=content)
                     else:
                         # It's a regular function - call it directly
-                        result = self.active_executor(step_input)  # type: ignore
+                        result = self._call_custom_function(self.active_executor, step_input, session_state_copy)  # type: ignore
                         if isinstance(result, StepOutput):
                             final_response = result
                         else:
                             final_response = StepOutput(content=str(result))
+
+                    # Merge session_state changes back
+                    if session_state is not None:
+                        merge_dictionaries(session_state, session_state_copy)
                 else:
                     # For agents and teams, prepare message with context
                     message = self._prepare_message(
@@ -741,9 +954,22 @@ class Step:
                         if isinstance(self.active_executor, Team):
                             kwargs["store_member_responses"] = True
 
-                        session_state_copy = copy(session_state)
+                        num_history_runs = self.num_history_runs if self.num_history_runs else num_history_runs
+
+                        use_history = (
+                            self.add_workflow_history
+                            if self.add_workflow_history is not None
+                            else add_workflow_history_to_steps
+                        )
+
+                        final_message = message
+                        if use_history and workflow_session:
+                            history_messages = workflow_session.get_workflow_history_context(num_runs=num_history_runs)
+                            if history_messages:
+                                final_message = f"{history_messages}{message}"
+
                         response_stream = self.active_executor.arun(  # type: ignore
-                            input=message,
+                            input=final_message,
                             images=images,
                             videos=videos,
                             audio=audios,
@@ -753,33 +979,28 @@ class Step:
                             session_state=session_state_copy,
                             stream=True,
                             stream_intermediate_steps=stream_intermediate_steps,
-                            # Pass workflow context directly via kwargs
-                            workflow_context={
-                                "workflow_id": workflow_run_response.workflow_id if workflow_run_response else None,
-                                "workflow_run_id": workflow_run_response.run_id if workflow_run_response else None,
-                                "step_id": self.step_id,
-                                "step_name": self.name,
-                                "step_index": step_index,
-                            },
                             yield_run_response=True,
                             **kwargs,
                         )
 
                         active_executor_run_response = None
                         async for event in response_stream:
-                            log_debug(f"Received async event from agent: {type(event).__name__}")
                             if isinstance(event, RunOutput) or isinstance(event, TeamRunOutput):
                                 active_executor_run_response = event
                                 break
-                            yield event  # type: ignore[misc]
+                            enriched_event = self._enrich_event_with_context(event, workflow_run_response, step_index)
+                            # Only yield executor events if stream_executor_events is True
+                            if stream_executor_events:
+                                yield enriched_event  # type: ignore[misc]
 
-                        # Update workflow session state
-                        merge_dictionaries(session_state, session_state_copy)  # type: ignore
+                        if session_state is not None:
+                            # Update workflow session state
+                            merge_dictionaries(session_state, session_state_copy)  # type: ignore
 
                         if store_executor_outputs and workflow_run_response is not None:
                             self._store_executor_response(workflow_run_response, active_executor_run_response)  # type: ignore
 
-                        final_response = self._process_step_output(active_executor_run_response)  # type: ignore
+                        final_response = active_executor_run_response  # type: ignore
                     else:
                         raise ValueError(f"Unsupported executor type: {self._executor_type}")
 
@@ -791,6 +1012,7 @@ class Step:
                 use_workflow_logger()
 
                 # Yield the final response
+                final_response = self._process_step_output(final_response)
                 yield final_response
 
                 if stream_intermediate_steps and workflow_run_response:
@@ -835,6 +1057,14 @@ class Step:
             executor_run_response.parent_run_id = workflow_run_response.run_id
             executor_run_response.workflow_step_id = self.step_id
 
+            # Scrub the executor response based on the executor's storage flags before storing
+            if (
+                not self.active_executor.store_media
+                or not self.active_executor.store_tool_messages
+                or not self.active_executor.store_history_messages
+            ):  # type: ignore
+                self.active_executor._scrub_run_output_for_storage(executor_run_response)  # type: ignore
+
             # Get the raw response from the step's active executor
             raw_response = executor_run_response
             if raw_response and isinstance(raw_response, (RunOutput, TeamRunOutput)):
@@ -1021,3 +1251,28 @@ class Step:
                 continue
 
         return videos
+
+
+def _is_async_callable(obj: Any) -> TypeGuard[Callable[..., Any]]:
+    """Checks if obj is an async callable (coroutine function or callable with async __call__)"""
+    return inspect.iscoroutinefunction(obj) or (callable(obj) and inspect.iscoroutinefunction(obj.__call__))
+
+
+def _is_generator_function(obj: Any) -> TypeGuard[Callable[..., Any]]:
+    """Checks if obj is a generator function, including callable class instances with generator __call__ methods"""
+    if inspect.isgeneratorfunction(obj):
+        return True
+    # Check if it's a callable class instance with a generator __call__ method
+    if callable(obj) and hasattr(obj, "__call__"):
+        return inspect.isgeneratorfunction(obj.__call__)
+    return False
+
+
+def _is_async_generator_function(obj: Any) -> TypeGuard[Callable[..., Any]]:
+    """Checks if obj is an async generator function, including callable class instances"""
+    if inspect.isasyncgenfunction(obj):
+        return True
+    # Check if it's a callable class instance with an async generator __call__ method
+    if callable(obj) and hasattr(obj, "__call__"):
+        return inspect.isasyncgenfunction(obj.__call__)
+    return False
